"""Tests for the AGT version gate."""

from __future__ import annotations

from aria_agentkit.compat import AGT_VERSION


class TestCompat:
    def test_agt_version_is_3x(self) -> None:
        assert AGT_VERSION[0] == 3

    def test_agt_version_is_tuple(self) -> None:
        assert isinstance(AGT_VERSION, tuple)
        assert len(AGT_VERSION) == 3

    def test_agt_version_parts_are_ints(self) -> None:
        for part in AGT_VERSION:
            assert isinstance(part, int)
